/* tslint:disable */
/* eslint-disable */
export const memory: WebAssembly.Memory;
export function __wbg_tiktoken_free(a: number): void;
export function tiktoken_new(a: number, b: number, c: number, d: number, e: number): number;
export function tiktoken_name(a: number, b: number): void;
export function tiktoken_encode(a: number, b: number, c: number, d: number, e: number, f: number): void;
export function tiktoken_encode_ordinary(a: number, b: number, c: number, d: number): void;
export function tiktoken_encode_with_unstable(a: number, b: number, c: number, d: number, e: number, f: number): void;
export function tiktoken_encode_single_token(a: number, b: number, c: number): number;
export function tiktoken_decode(a: number, b: number, c: number, d: number): void;
export function tiktoken_decode_single_token_bytes(a: number, b: number, c: number): void;
export function tiktoken_token_byte_values(a: number): number;
export function get_encoding(a: number, b: number, c: number, d: number): void;
export function encoding_for_model(a: number, b: number, c: number, d: number): void;
export function __wbindgen_export_0(a: number): number;
export function __wbindgen_export_1(a: number, b: number, c: number): number;
export function __wbindgen_add_to_stack_pointer(a: number): number;
export function __wbindgen_export_2(a: number, b: number): void;
export function __wbindgen_export_3(a: number): void;
